@@ -1,18 +1,22 @@
 use hir::Semantics;
 use ide_db::{
     base_db::FilePosition,
-    defs::Definition,
+    defs::{Definition, NameClass, NameRefClass},
     helpers::{for_each_break_expr, for_each_tail_expr, pick_best_token},
     search::{FileReference, ReferenceAccess, SearchScope},
     RootDatabase,
 };
+use rustc_hash::FxHashSet;
 use syntax::{
     ast::{self, LoopBodyOwner},
-    match_ast, AstNode, SyntaxNode, SyntaxToken, TextRange, T,
+    match_ast, AstNode,
+    SyntaxKind::IDENT,
+    SyntaxNode, SyntaxToken, TextRange, TextSize, T,
 };
 
 use crate::{display::TryToNav, references, NavigationTarget};
 
+#[derive(PartialEq, Eq, Hash)]
 pub struct HighlightedRange {
     pub range: TextRange,
     pub access: Option<ReferenceAccess>,
@@ -44,9 +48,10 @@ pub(crate) fn highlight_related(
     let syntax = sema.parse(position.file_id).syntax().clone();
 
     let token = pick_best_token(syntax.token_at_offset(position.offset), |kind| match kind {
-        T![?] => 3, // prefer `?` when the cursor is sandwiched like in `await$0?`
-        T![->] => 2,
-        kind if kind.is_keyword() => 1,
+        T![?] => 4, // prefer `?` when the cursor is sandwiched like in `await$0?`
+        T![->] => 3,
+        kind if kind.is_keyword() => 2,
+        IDENT => 1,
         _ => 0,
     })?;
 
@@ -70,35 +75,41 @@ fn highlight_references(
     syntax: &SyntaxNode,
     FilePosition { offset, file_id }: FilePosition,
 ) -> Option<Vec<HighlightedRange>> {
-    let def = references::find_def(sema, syntax, offset)?;
-    let usages = def
-        .usages(sema)
-        .set_scope(Some(SearchScope::single_file(file_id)))
-        .include_self_refs()
-        .all();
+    let defs = find_defs(sema, syntax, offset);
+    let usages = defs
+        .iter()
+        .filter_map(|&d| {
+            d.usages(sema)
+                .set_scope(Some(SearchScope::single_file(file_id)))
+                .include_self_refs()
+                .all()
+                .references
+                .remove(&file_id)
+        })
+        .flatten()
+        .map(|FileReference { access, range, .. }| HighlightedRange { range, access });
 
-    let declaration = match def {
-        Definition::ModuleDef(hir::ModuleDef::Module(module)) => {
-            Some(NavigationTarget::from_module_to_decl(sema.db, module))
+    let declarations = defs.iter().flat_map(|def| {
+        match def {
+            &Definition::ModuleDef(hir::ModuleDef::Module(module)) => {
+                Some(NavigationTarget::from_module_to_decl(sema.db, module))
+            }
+            def => def.try_to_nav(sema.db),
         }
-        def => def.try_to_nav(sema.db),
-    }
-    .filter(|decl| decl.file_id == file_id)
-    .and_then(|decl| {
-        let range = decl.focus_range?;
-        let access = references::decl_access(&def, syntax, range);
-        Some(HighlightedRange { range, access })
+        .filter(|decl| decl.file_id == file_id)
+        .and_then(|decl| {
+            let range = decl.focus_range?;
+            let access = references::decl_access(&def, syntax, range);
+            Some(HighlightedRange { range, access })
+        })
     });
 
-    let file_refs = usages.references.get(&file_id).map_or(&[][..], Vec::as_slice);
-    let mut res = Vec::with_capacity(file_refs.len() + 1);
-    res.extend(declaration);
-    res.extend(
-        file_refs
-            .iter()
-            .map(|&FileReference { access, range, .. }| HighlightedRange { range, access }),
-    );
-    Some(res)
+    let res: FxHashSet<_> = declarations.chain(usages).collect();
+    if res.is_empty() {
+        None
+    } else {
+        Some(res.into_iter().collect())
+    }
 }
 
 fn highlight_exit_points(
@@ -265,6 +276,46 @@ fn cover_range(r0: Option<TextRange>, r1: Option<TextRange>) -> Option<TextRange
     }
 }
 
+fn find_defs(
+    sema: &Semantics<RootDatabase>,
+    syntax: &SyntaxNode,
+    offset: TextSize,
+) -> FxHashSet<Definition> {
+    sema.find_nodes_at_offset_with_descend(syntax, offset)
+        .flat_map(|name_like| {
+            Some(match name_like {
+                ast::NameLike::NameRef(name_ref) => {
+                    match NameRefClass::classify(sema, &name_ref)? {
+                        NameRefClass::Definition(def) => vec![def],
+                        NameRefClass::FieldShorthand { local_ref, field_ref } => {
+                            vec![Definition::Local(local_ref), Definition::Field(field_ref)]
+                        }
+                    }
+                }
+                ast::NameLike::Name(name) => match NameClass::classify(sema, &name)? {
+                    NameClass::Definition(it) | NameClass::ConstReference(it) => vec![it],
+                    NameClass::PatFieldShorthand { local_def, field_ref } => {
+                        vec![Definition::Local(local_def), Definition::Field(field_ref)]
+                    }
+                },
+                ast::NameLike::Lifetime(lifetime) => {
+                    NameRefClass::classify_lifetime(sema, &lifetime)
+                        .and_then(|class| match class {
+                            NameRefClass::Definition(it) => Some(it),
+                            _ => None,
+                        })
+                        .or_else(|| {
+                            NameClass::classify_lifetime(sema, &lifetime)
+                                .and_then(NameClass::defined)
+                        })
+                        .map(|it| vec![it])?
+                }
+            })
+        })
+        .flatten()
+        .collect()
+}
+
 #[cfg(test)]
 mod tests {
     use crate::fixture;
@@ -362,6 +413,45 @@ fn foo() {
         );
     }
 
+    #[test]
+    fn test_multi_macro_usage() {
+        check(
+            r#"
+macro_rules! foo {
+    ($ident:ident) => {
+        fn $ident() -> $ident { loop {} }
+        struct $ident;
+    }
+}
+
+foo!(bar$0);
+  // ^^^
+fn foo() {
+    let bar: bar = bar();
+          // ^^^
+                // ^^^
+}
+"#,
+        );
+        check(
+            r#"
+macro_rules! foo {
+    ($ident:ident) => {
+        fn $ident() -> $ident { loop {} }
+        struct $ident;
+    }
+}
+
+foo!(bar);
+  // ^^^
+fn foo() {
+    let bar: bar$0 = bar();
+          // ^^^
+}
+"#,
+        );
+    }
+
     #[test]
     fn test_hl_yield_points() {
         check(
@@ -773,6 +863,21 @@ fn foo() {
         );
     }
 
+    #[test]
+    fn test_hl_field_shorthand() {
+        check(
+            r#"
+struct Struct { field: u32 }
+              //^^^^^
+fn function(field: u32) {
+          //^^^^^
+    Struct { field$0 }
+           //^^^^^ read
+}
+"#,
+        );
+    }
+
     #[test]
     fn test_hl_disabled_ref_local() {
         let config = HighlightRelatedConfig {
